// set the dimensions and margins of the graph
const width = 800;
const height = 500;
const width1 = width * 0.5;
const margin = 60;

const svg = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

const happyScale = d3
  .scaleLinear()
  .domain([0, 10])
  .range([height - margin, margin]);

const gdpScale = d3
  .scaleLog()
  .domain([5, 13])
  .range([margin, width1 - margin]);

const svg2 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

const tooltip = d3
  .select('#my_dataviz')
  .append('div')
  .attr('id', 'tooltip');

const dropdown = d3.select('#continentSelect');

const continents = [
  'Asia',
  'Europe',
  'North America',
  'South America',
  'Oceania',
  'Africa',
  'Unknown',
];

dropdown
  .selectAll('[empty]')
  .data(['continentUrl', ...continents])
  .join('option')
  .attr('value', (d) => d)
  .text((d) => d);

svg2
  .append('text')
  .text('GDP Per Capita')
  .attr('x', width1 / 2)
  .attr('y', height - margin / 3)
  .style('text-anchor', 'middle');

svg2
  .append('text')
  .text('Happiness Score')
  .attr('transform', 'rotate(-90)')
  .attr('x', -height / 2)
  .attr('y', margin / 2);

svg2
  .append('g')
  .attr('transform', `translate(0,${height - margin})`) // This controls the vertical position of the Axis
  .call(d3.axisBottom(gdpScale));

svg2
  .append('g')
  .attr('transform', `translate(${margin},0)`) // This controls the vertical position of the Axis
  .call(d3.axisRight(happyScale));

function render(mapData, countryData) {
  var projection = d3
    .geoEqualEarth()
    .fitSize([width / 2, height], mapData);
  var path = d3.geoPath(projection);

  const mapColorAttr = 'Ladder score';
  var mapColorScale = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          countryData.values(),
          (r) => +r[mapColorAttr],
        ),
      ),
    )
    .range(['white', 'blue']);

  function getCountryColor(name) {
    let entry = countryData.get(name);
    if (!(entry == undefined)) {
      return mapColorScale(entry[mapColorAttr]);
    }
    return 'grey';
  }

  const xAttr = 'Logged GDP per capita';
  const x = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(countryData.values(), (d) => +d[xAttr]),
      ),
    )
    .range([0, width / 2]);
  console.log(x.domain());

  const yAttr = 'Ladder score';
  const y = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          countryData.values().filter((d) => +d[yAttr] > 0),
          (d) => d[yAttr],
        ),
      ),
    )
    .range([height, 0]);

  const plotColorAttr = 'Continent';
  const plotColorScale = d3.scaleOrdinal(
    d3.schemeCategory10,
  );
  function getPointColor(d) {
    console.log(d);
    return plotColorScale(d[plotColorAttr]);
  }

  svg2
    .selectAll('[empty]')
    .data(continents)
    .join('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', (d) => plotColorScale(d))
    .attr('stroke', 'black')
    .attr('x', margin * 1.5)
    .attr('y', (d) => margin + 20 * continents.indexOf(d));

  svg2
    .selectAll('[empty]')
    .data(continents)
    .join('text')
    .text((d) => d)
    .attr('x', margin * 1.5 + 32)
    .attr(
      'y',
      (d) => margin + 16 + 20 * continents.indexOf(d),
    );

  function popUpTooltip(event, data) {
    let country = countryData.get(data.properties.name);
    tooltip
      .style('opacity', 1)
      .style('z-index', 10)
      .html(
        `${country['Country name']}<br/>Happy Score: ${country['Ladder score']}<br/>GDP: ${country['Logged GDP per capita']}`,
      );
  }

  function adjustTooltip(event) {
    tooltip
      .style('left', event.x - 80 + 'px')
      .style('top', event.y - 160 + 'px');
  }

  const rAttr = 'Ladder score';
  const rScale = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(countryData.values(), (d) => d[rAttr]),
      ),
    )
    .range([4, 4]);

  const countryMarks = svg
    .append('g')
    .selectAll('path')
    .data(mapData.features)
    .join('path')
    .attr('fill', (d) => getCountryColor(d.properties.name))
    .attr('id', (d) => d.properties.name)
    .classed('poly', true)
    .attr('d', path);

  countryMarks
    .on('mouseover', function (event, data) {
      popUpTooltip(event, data);
      d3.selectAll(`#${this.id}`)
        .attr('fill', 'red')
        .attr('r', 15)
        .selectAll('circle');
    })
    .on('mousemove', function (event, data) {
      adjustTooltip(event);
    })
    .on('mouseleave', function (event, data) {
      tooltip.style('opacity', 0).style('z-index', -1);
      d3.selectAll(`circle#${this.id}`)
        .attr(
          'fill',
          plotColorScale(
            countryData.get(data.properties.name).Continent,
          ),
        )
        .attr('r', (d) => rScale(d[rAttr]));
      d3.selectAll(`path#${this.id}`)
        .attr('fill', getCountryColor(data.properties.name))
        .attr('r', (d) => rScale(d[rAttr]));
    });

  svg2
    .append('g')
    .selectAll('circle')
    .data(countryData.values().filter((c) => +c[yAttr] > 0))
    .join('circle')
    .attr('cx', (d) => gdpScale(d[xAttr]))
    .attr('cy', (d) => happyScale(d[yAttr]))
    .attr('r', (d) => rScale(d[rAttr]))
    .attr('fill', getPointColor)
    .attr('id', (d) => d['Country name'])
    .on('mouseover', function (event, data) {
      //d3.select(this).attr('fill', 'red');
      d3.selectAll(`#${this.id}`)
        .attr('fill', 'red')
        .attr('r', 15)
        .selectAll('circle');
    })
    .on('mouseleave', function (event, data) {
      console.log(data);
      d3.selectAll(`circle#${this.id}`)
        .attr('fill', getPointColor(data))
        .attr('r', (d) => rScale(d[rAttr]));
      d3.selectAll(`path#${this.id}`)
        .attr('fill', getCountryColor(data['Country name']))
        .attr('r', (d) => rScale(d[rAttr]));
    });
}

//Read the data
const mapUrl =
  'https://raw.githubusercontent.com/holtzy/D3-graph-gallery/master/DATA/world.geojson';
const countryStatsUrl =
  'https://raw.githubusercontent.com/KaijaVZ/happyData/main/WHR2023.csv';
const continentUrl =
  'https://raw.githubusercontent.com/samayo/country-json/master/src/country-by-continent.json';

function normalizeCountryName(s) {
  return s.replaceAll(' ', '_').toLowerCase();
}

Promise.all([
  d3.json(mapUrl),
  d3.csv(countryStatsUrl),
  d3.json(continentUrl),
]).then(([mapData, csvData, continentData]) => {
  var countryData = new Map();
  csvData.forEach((row) => {
    console.log(row);
    row.Continent = 'Unknown';
    row['Country name'] = normalizeCountryName(
      row['Country name'],
    );

    countryData.set(row['Country name'], row);
  });
  mapData.features.forEach((f) => {
    f.properties.name = normalizeCountryName(
      f.properties.name,
    );
  });
  continentData.forEach((row) => {
    let name = normalizeCountryName(row.country);
    if (countryData.has(name)) {
      countryData.get(name).Continent = row.continent;
    } else {
      console.log('No match found for ', name);
    }
  });
  console.log(countryData);
  render(mapData, countryData);
});
